/*****************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002-2006 Xilinx Inc.
*       All rights reserved.
*
*******************************************************************************/

/*******************************************************************************
** XAPP967: Creating an OPB IPIF Based IP and using it in an EDK System
** File: system.c
** Vesrion 1.0
** Version: Release Version 1.0: 21/11/2006
** Software Version: Xilinx EDK Release 7.1 SP#2
** August 2005
** Target Board:  ML403 Board
** FPGA Family:	 Virtex-4 FX
** Processor: PowerPC CPU
** Debug interface: FPGA JTAG Interface
*******************************************************************************/

#include "xintc.h"
#include "xexception_l.h"
#include "xparameters.h"
#include "opb_ipif_template.h"

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */

#define UARTLITE_DEVICE_ID      XPAR_RS232_UART_DEVICE_ID
#define INTC_DEVICE_ID          XPAR_OPB_INTC_0_DEVICE_ID
#define IPIF_INT_IRQ_ID         XPAR_OPB_INTC_0_OPB_IPIF_TEMPLATE_0_IP2INTC_IRPT_INTR

/*
 * The following constant controls the length of the buffers to be used 
 * for the Data Packet, we are using 1024 Words = 4096 Bytes
 */
#define TEST_BUFFER_SIZE       4096

/************************** Function Prototypes ******************************/

void My_ip_Interrupt_Handler(void * baseaddr_p);

/*
 * The following buffers are used in this example to send and receive data
 * with the IPIF Template IP.
 */
Xuint8 *SendBuffer;
Xuint8 *ReceiveBuffer;

/*
 * The following Global Variable is used as a shared Global Variable between main()
 * and the OPB IPIF Template IP Interrupt Hanlder
 */
Xuint32 Rx_Irq;

XIntc InterruptController;     /* The instance of the Interrupt Controller */

int main(void)
   {

XStatus Status;
Xuint32 Index;

// Initializing the Send Buffer to the OPB BRAM 1:
SendBuffer = (Xuint8 *)XPAR_OPB_BRAM_IF_CNTLR_1_BASEADDR;

// Initializing the Receive Buffer to the OPB BRAM 2:
ReceiveBuffer = (Xuint8 *)XPAR_OPB_BRAM_IF_CNTLR_2_BASEADDR;

// Reset the Shared Global Variable:
Rx_Irq = 0x0;

// Print the Application Header to the UART    
   print("\n\r   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
	print("\n\r   +                                                                 +");
	print("\n\r   +                       Xilinx ML403 Board                        +");
	print("\n\r   +                            XAPP 967:                            +");
	print("\n\r   +   Creating an OPB IPIF Based IP and using it in an EDK System   +");
	print("\n\r   +                                                                 +");
	print("\n\r   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n\r"); 


    /*
     * Initialize the interrupt controller driver so that it is ready to use
     */
    Status = XIntc_Initialize(&InterruptController, INTC_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        print("\n\r --> ERROR Initializing the OPB Interrupt Controller...");
		  return XST_FAILURE;
    }
    
	 print("\n\r --> Initializing the OPB Interrupt Controller returned No Errors");


    /*
     * Connect a device driver handler that will be called when an interrupt
     * for the device occurs, the device driver handler performs the specific
     * interrupt processing for the device
     */
    
	 Status = XIntc_Connect(&InterruptController, XPAR_OPB_INTC_0_OPB_IPIF_TEMPLATE_0_IP2INTC_IRPT_INTR,
                           (XInterruptHandler)My_ip_Interrupt_Handler,
                           (void *)XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR);
    if (Status != XST_SUCCESS)
    {
        print("\n\r --> ERROR Connecting the OPB IPIF Template Interrupt Handler...");
		  return XST_FAILURE;
    }
    
	 print("\n\r --> Connecting the OPB IPIF Template Interrupt Handler returned No Errors");

	 
    /*
     * Start the interrupt controller such that interrupts are enabled for
     * all devices that cause interrupts, specific real mode so that
     * the OPB IPIF Template IP can cause interrupts thru the interrupt controller.
     */
    Status = XIntc_Start(&InterruptController, XIN_REAL_MODE);
    if (Status != XST_SUCCESS)
    {
        print("\n\r --> ERROR Starting the OPB Interrupt Controller...");
		  return XST_FAILURE;
    }
    
	 print("\n\r --> The OPB Interrupt Controller Started with No Errors");


    /*
     * Enable the interrupt for the OPB IPIF Template
     */
    XIntc_Enable(&InterruptController, IPIF_INT_IRQ_ID);

    /*
     * Initialize the PPC405 exception table
     */
    XExc_Init();

    /*
     * Register the interrupt controller handler with the exception table
     */
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,
                         (XExceptionHandler)XIntc_InterruptHandler,
                         &InterruptController);

    /*
     * Enable non-critical exceptions
     */
    XExc_mEnableExceptions(XEXC_NON_CRITICAL);

    /*
     * Enable the interrupt of the OPB IPIF Template so interrupts will occur.
     */
    OPB_IPIF_TEMPLATE_EnableInterrupt((void *)XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR);


    /*
     * Initialize the send buffer bytes with a pattern to send and the
     * the receive buffer bytes to zero to allow the receive data to be
     * verified
     */
    for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
        SendBuffer[Index]   = 0xA5;
		  Index++;
		  SendBuffer[Index] = 0xB6;
		  Index++;
		  SendBuffer[Index] = 0xC7;
		  Index++;
		  SendBuffer[Index] = 0xD8;
    }
	 
	 
	 for (Index = 0; Index < TEST_BUFFER_SIZE; Index++)
    {
		  ReceiveBuffer[Index] = 0;
    }
   
   
	xil_printf("\n\r   - Reset DMA channel 0 to initial state \n\r");
	OPB_IPIF_TEMPLATE_mResetDMA0(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR);
	xil_printf("   - Setup DMA channel 0 to transfer data from remote source to local FIFO \n\r");
   OPB_IPIF_TEMPLATE_mSetDMA0Control(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR, DMA_SINC_MASK | DMA_DLOCAL_MASK);

	// Post 4 KB of Data to the Tx FIFO:

	 xil_printf("   - Start DMA channel 0 to receive bytes from the source buffer \n\r");
   
	 OPB_IPIF_TEMPLATE_DMA0Transfer(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR, (Xuint32) SendBuffer, XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR+OPB_IPIF_TEMPLATE_WRFIFO_DATA_OFFSET, TEST_BUFFER_SIZE);
   
	 while ( ! OPB_IPIF_TEMPLATE_mDMA0Done(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR) ) {}
   
	 xil_printf("   - DMA channel 0 transfer completed \n\r");
   
	 if ( OPB_IPIF_TEMPLATE_mDMA0Error(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR) )
    {
      xil_printf("   - DMA channel 0 transfer error \n\r");
      xil_printf("   - DMA failed \n\r");
      return XST_FAILURE;
    }  
	
   // Pending for the OPB IPIF Receive Interrupt to occur:
   while(Rx_Irq != 0xFFFFFFFF);	 

   xil_printf("   - Reset DMA channel 1 to initial state \n\r");
   
	OPB_IPIF_TEMPLATE_mResetDMA1(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR);
   
	xil_printf("   - Setup DMA channel 1 to transfer data from local FIFO to remote destination \n\r");
   
	OPB_IPIF_TEMPLATE_mSetDMA1Control(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR, DMA_DINC_MASK | DMA_SLOCAL_MASK);
   
	xil_printf("   - Start DMA channel 1 to send bytes to the destination buffer \n\r");
	
	 OPB_IPIF_TEMPLATE_DMA1Transfer(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR, XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR+OPB_IPIF_TEMPLATE_RDFIFO_DATA_OFFSET, (Xuint32) ReceiveBuffer, TEST_BUFFER_SIZE);
   
	 while ( ! OPB_IPIF_TEMPLATE_mDMA1Done(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR) ) {}
   
	 xil_printf("   - DMA channel 1 transfer completed \n\r");
   
	 if ( OPB_IPIF_TEMPLATE_mDMA1Error(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR) )
    {
      xil_printf("   - DMA channel 1 transfer error \n\r");
      xil_printf("   - DMA failed \n\r");
      return XST_FAILURE;
    }
	
	
	for ( Index = 0; Index < TEST_BUFFER_SIZE; Index++ )
   {
		if ( (ReceiveBuffer[Index] != 0xA5) || (ReceiveBuffer[Index+1] != 0xB6) ||(ReceiveBuffer[Index+2] != 0xC7) ||(ReceiveBuffer[Index+3] != 0xD8) )
       {
        xil_printf("   - Destination buffer's contents are different from the source buffer \n\r");
		  putnum(Index);
        xil_printf("   - DMA failed \n\r");
        return XST_FAILURE;
       }
		 Index= Index+3;
   }
   
	xil_printf("   - Destination Buffer's contents match the Send Buffer's Contents  \n\r");
   OPB_IPIF_TEMPLATE_mResetDMA0(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR);
   OPB_IPIF_TEMPLATE_mResetDMA1(XPAR_OPB_IPIF_TEMPLATE_0_BASEADDR);	
	
	print("\n\r End of main()");
	
	return 0;
  } 
// End of main()

/**
 * Interrupt Handler of the OPB IPIF Template IP
 * 
 */
void My_ip_Interrupt_Handler(void * baseaddr_p)
{
  Xuint32 baseaddr;
  Xuint32 IntrStatus;
  Xuint32 IpStatus;

  baseaddr = (Xuint32) baseaddr_p;

   /*
   * Reset global interrupt enable Register.
   */
  OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DGIER_OFFSET, 0x0);
  
  /*
   * Get status from Device Interrupt Status Register.
   */
  IntrStatus = OPB_IPIF_TEMPLATE_mReadReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DISR_OFFSET);

  xil_printf("\n\r  - Device Interrupt! DISR value : 0x%08x \n\r", IntrStatus);

  /*
   * Verify the source of the interrupt is the user logic and clear the interrupt
   * source by toggle write back to the IP ISR register.
   */
  if ( (IntrStatus & INTR_IPIR_MASK) == INTR_IPIR_MASK )
  {
    xil_printf("\n\r   - User logic interrupt! \n\r");
    IpStatus = OPB_IPIF_TEMPLATE_mReadReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET);
    OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET, IpStatus);
  }

 // Acknowledge the Rx Interrupt from the User Logic to main():
 Rx_Irq = 0xFFFFFFFF; 

}

