//////////////////////////////////////////////////////////////////////////////
// Filename:          C:\xapp967/drivers/opb_ipif_template_v1_00_a/src/opb_ipif_template.c
// Version:           1.00.a
// Description:       opb_ipif_template Driver Source File
// Date:              Thu Oct 19 19:24:20 2006 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////


/***************************** Include Files *******************************/

#include "opb_ipif_template.h"

/************************** Function Definitions ***************************/


/**
 *
 * DMA transfer between source address and destination address, either the source or the destination address.
 * must be local to the OPB_IPIF_TEMPLATE device.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 * @param   SrcAddress is the source address from which the data will be fetched.
 * @param   DstAddress is the destination address to which the data will be stored.
 * @param   ByteCount is the number of bytes to be sent.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void OPB_IPIF_TEMPLATE_DMA0Transfer(Xuint32 BaseAddress, Xuint32 SrcAddress, Xuint32 DstAddress, Xuint32 ByteCount)
{
  /*
   * Setup source address and destination address for transfer.
   */
  XIo_Out32(BaseAddress+OPB_IPIF_TEMPLATE_DMA0_SA_OFFSET, SrcAddress);
  XIo_Out32(BaseAddress+OPB_IPIF_TEMPLATE_DMA0_DA_OFFSET, DstAddress);

  /*
   * Start the DMA transfer to copy from the source buffer to the destination buffer
   * by writing the length to the Length register.
   */
  XIo_Out32(BaseAddress+OPB_IPIF_TEMPLATE_DMA0_LENGTH_OFFSET, ByteCount);
}

void OPB_IPIF_TEMPLATE_DMA1Transfer(Xuint32 BaseAddress, Xuint32 SrcAddress, Xuint32 DstAddress, Xuint32 ByteCount)
{
  /*
   * Setup source address and destination address for transfer.
   */
  XIo_Out32(BaseAddress+OPB_IPIF_TEMPLATE_DMA1_SA_OFFSET, SrcAddress);
  XIo_Out32(BaseAddress+OPB_IPIF_TEMPLATE_DMA1_DA_OFFSET, DstAddress);

  /*
   * Start the DMA transfer to copy from the source buffer to the destination buffer
   * by writing the length to the Length register.
   */
  XIo_Out32(BaseAddress+OPB_IPIF_TEMPLATE_DMA1_LENGTH_OFFSET, ByteCount);
}

/**
 *
 * Enable all possible interrupts from OPB_IPIF_TEMPLATE device.
 *
 * @param   baseaddr_p is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void OPB_IPIF_TEMPLATE_EnableInterrupt(void * baseaddr_p)
{
  Xuint32 baseaddr;
  baseaddr = (Xuint32) baseaddr_p;

  /*
   * Enable all interrupt source from user logic.
   */
  OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_IER_OFFSET, 0x00000001);

  /*
   * Enable all possible interrupt sources from device.
   */
  OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DIER_OFFSET,
    INTR_TERR_MASK
    | INTR_DPTO_MASK
    | INTR_IPIR_MASK
    | INTR_DMA0_MASK
    | INTR_DMA1_MASK
    | INTR_RFDL_MASK
    | INTR_WFDL_MASK
    );

  /*
   * Set global interrupt enable.
   */
  OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DGIER_OFFSET, INTR_GIE_MASK);
}

/**
 *
 * Example interrupt controller handler for OPB_IPIF_TEMPLATE device.
 * This is to show example of how to toggle write back ISR to clear interrupts.
 *
 * @param   baseaddr_p is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void OPB_IPIF_TEMPLATE_Intr_DefaultHandler(void * baseaddr_p)
{
  Xuint32 baseaddr;
  Xuint32 IntrStatus;
Xuint32 IpStatus;

  baseaddr = (Xuint32) baseaddr_p;

  /*
   * Get status from Device Interrupt Status Register.
   */
  IntrStatus = OPB_IPIF_TEMPLATE_mReadReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DISR_OFFSET);

  xil_printf("Device Interrupt! DISR value : 0x%08x \n\r", IntrStatus);

  /*
   * Verify the source of the interrupt is the user logic and clear the interrupt
   * source by toggle write baca to the IP ISR register.
   */
  if ( (IntrStatus & INTR_IPIR_MASK) == INTR_IPIR_MASK )
  {
    xil_printf("User logic interrupt! \n\r");
    IpStatus = OPB_IPIF_TEMPLATE_mReadReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET);
    OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET, IpStatus);
  }

}

// CUSTOM FUNCTIONS DEFINITION:
/**
 * A Function that will reset the Custom IP, Write 1024 Bytes to the Write FIFO using Channel 0 DMA, is written 
 *  and added to the drivers.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   Xuint32 SrcAddress is the address of the buffer memory where Data is located.
 * 
 * @return 
 *
 *    - XST_SUCCESS   if the transfer is OK.
 *    - Service Error Code is not.
 *
 */

XStatus OPB_IPIF_TEMPLATE_Write1KB(Xuint32 SrcAddress, Xuint32 BaseAddress)
{
Xuint32 pfifo_status = 0;
Xuint32 IPIF_Temp_B_Addr;
Xuint32 pfifo_vacancy;
Xuint32 dma_ctrl_mask;
Xuint32 dma_status;

IPIF_Temp_B_Addr = BaseAddress;
// Reset the OPB IPIF Template IP:
OPB_IPIF_TEMPLATE_mReset(IPIF_Temp_B_Addr);

// Check status of OPB_IPIF_TEMPLATE write packet FIFO module:
pfifo_status = OPB_IPIF_TEMPLATE_mWriteFIFOFull(IPIF_Temp_B_Addr);
if (pfifo_status == 1 )
{
return WRFIFO_FULL_MASK; 
}

// Check that the Vacancy (in Words) of the Write FIFO can hold the 1KB of Data:
pfifo_vacancy = OPB_IPIF_TEMPLATE_mWriteFIFOVacancy(IPIF_Temp_B_Addr);

if (pfifo_vacancy < 1024/4 )
{
return WRFIFO_FULL_MASK; 
}

// Reset the Channel 0 DMA:
OPB_IPIF_TEMPLATE_mResetDMA0(IPIF_Temp_B_Addr);

// Setup the Channel 0 DMA Control Register:
// - The Source Address is a Memory Buffer, therefore the Source Address should be incremented.
// - The Destination Address is Local since it is the Write FIFO.

dma_ctrl_mask = DMA_SINC_MASK | DMA_DLOCAL_MASK; 

OPB_IPIF_TEMPLATE_mSetDMA0Control(IPIF_Temp_B_Addr, dma_ctrl_mask);

// Set the Source Address and Start the DMA Transfer.
// The destination address is local to the OPB_IPIF_TEMPLATE device:
// In this case it is the Write Packet FIFO:
OPB_IPIF_TEMPLATE_DMA0Transfer(IPIF_Temp_B_Addr, SrcAddress, 0, 1024);

// Check if the DMA Transfer has finished: 
dma_status = OPB_IPIF_TEMPLATE_mDMA0Done(IPIF_Temp_B_Addr);
while(dma_status != 1 );

// Check if the DMA Transfer was achieved with No Errors:
dma_status =  OPB_IPIF_TEMPLATE_mDMA0Error(IPIF_Temp_B_Addr);

if ( dma_status == 1)
{
return DMA_DBE_MASK;
}

return XST_SUCCESS;

}



