//////////////////////////////////////////////////////////////////////////////
// Filename:          C:\xapp967/drivers/opb_ipif_template_v1_00_a/src/opb_ipif_template.h
// Version:           1.00.a
// Description:       opb_ipif_template Driver Header File
// Date:              Thu Oct 19 19:24:20 2006 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////

#ifndef OPB_IPIF_TEMPLATE_H
#define OPB_IPIF_TEMPLATE_H

/***************************** Include Files *******************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"

/************************** Constant Definitions ***************************/


/**
 * User Logic Slave Space Offsets
 */
#define OPB_IPIF_TEMPLATE_USER_SLAVE_SPACE_OFFSET (0x00000000)

/**
 * IPIF Reset/Mir Space Register Offsets
 * -- RST : software reset register
 * -- MIR : module identification register
 */
#define OPB_IPIF_TEMPLATE_IPIF_RST_SPACE_OFFSET (0x00000100)
#define OPB_IPIF_TEMPLATE_RST_OFFSET (OPB_IPIF_TEMPLATE_IPIF_RST_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_MIR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_RST_SPACE_OFFSET + 0x00000000)

/**
 * IPIF Reset/Mir Masks
 * -- IPIF_MAVN_MASK   : module major version number
 * -- IPIF_MIVN_MASK   : module minor version number
 * -- IPIF_MIVL_MASK   : module minor version letter
 * -- IPIF_BID_MASK    : module block id
 * -- IPIF_BTP_MASK    : module block type
 * -- IPIF_RESET       : software reset
 */
#define IPIF_MAVN_MASK (0xF0000000UL)
#define IPIF_MIVN_MASK (0x0FE00000UL)
#define IPIF_MIVL_MASK (0x001F0000UL)
#define IPIF_BID_MASK (0x0000FF00UL)
#define IPIF_BTP_MASK (0x000000FFUL)
#define IPIF_RESET (0x0000000A)

/**
 * IPIF Interrupt Controller Space Offsets
 * -- INTR_DISR  : device (ipif) interrupt status register
 * -- INTR_DIPR  : device (ipif) interrupt pending register
 * -- INTR_DIER  : device (ipif) interrupt enable register
 * -- INTR_DIIR  : device (ipif) interrupt id (priority encoder) register
 * -- INTR_DGIER : device (ipif) global interrupt enable register
 * -- INTR_ISR   : ip (user logic) interrupt status register
 * -- INTR_IER   : ip (user logic) interrupt enable register
 */
#define OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET (0x00000200)
#define OPB_IPIF_TEMPLATE_INTR_DISR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_INTR_DIPR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x00000004)
#define OPB_IPIF_TEMPLATE_INTR_DIER_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x00000008)
#define OPB_IPIF_TEMPLATE_INTR_DIIR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x00000018)
#define OPB_IPIF_TEMPLATE_INTR_DGIER_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x0000001C)
#define OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x00000020)
#define OPB_IPIF_TEMPLATE_INTR_IER_OFFSET (OPB_IPIF_TEMPLATE_IPIF_INTR_SPACE_OFFSET + 0x00000028)

/**
 * IPIF Interrupt Controller Masks
 * -- INTR_TERR_MASK : transaction error
 * -- INTR_DPTO_MASK : data phase time-out
 * -- INTR_IPIR_MASK : ip interrupt requeset
 * -- INTR_DMA0_MASK : dma channel 0 interrupt request
 * -- INTR_DMA1_MASK : dma channel 1 interrupt request
 * -- INTR_RFDL_MASK : read packet fifo deadlock interrupt request
 * -- INTR_WFDL_MASK : write packet fifo deadlock interrupt request
 * -- INTR_IID_MASK  : interrupt id
 * -- INTR_GIE_MASK  : global interrupt enable
 * -- INTR_NOPEND    : the DIPR has no pending interrupts
 */
#define INTR_TERR_MASK (0x00000001UL)
#define INTR_DPTO_MASK (0x00000002UL)
#define INTR_IPIR_MASK (0x00000004UL)
#define INTR_DMA0_MASK (0x00000008UL)
#define INTR_DMA1_MASK (0x00000010UL)
#define INTR_RFDL_MASK (0x00000020UL)
#define INTR_WFDL_MASK (0x00000040UL)
#define INTR_IID_MASK (0x000000FFUL)
#define INTR_GIE_MASK (0x80000000UL)
#define INTR_NOPEND (0x80)

/**
 * IPIF DMA & Scatter Gather Space Offsets
 * -- DMA0_RST    : dma & scatter gather channel 0 reset register
 * -- DMA0_MIR    : dma & scatter gather channel 0 module identification register
 * -- DMA0_DMACR  : dma & scatter gather channel 0 control register
 * -- DMA0_SA     : dma channel 0 source address register
 * -- DMA0_DA     : dma channel 0 destination address register
 * -- DMA0_LENGTH : dma channel 0 start/length register
 * -- DMA0_DMASR  : dma / scatter gather channel 0 status register
 * -- DMA0_BDA    : scatter gather channel 0 buffer descriptor address register
 * -- DMA0_SWCR   : scatter gather channel 0 software control register
 * -- DMA0_UPC    : scatter gather channel 0 unserviced packet count register
 * -- DMA0_PCT    : scatter gather channel 0 packet count threshold register
 * -- DMA0_PWB    : scatter gather channel 0 packet wait bound register
 * -- DMA0_ISR    : dma / scatter gather channel 0 interrupt status register
 * -- DMA0_IER    : dma / scatter gather channel 0 interrupt enable register
 * -- DMA1_RST    : dma & scatter gather channel 1 reset register
 * -- DMA1_MIR    : dma & scatter gather channel 1 module identification register
 * -- DMA1_DMACR  : dma & scatter gather channel 1 control register
 * -- DMA1_SA     : dma channel 1 source address register
 * -- DMA1_DA     : dma channel 1 destination address register
 * -- DMA1_LENGTH : dma channel 1 start/length register
 * -- DMA1_DMASR  : dma / scatter gather channel 1 status register
 * -- DMA1_BDA    : scatter gather channel 1 buffer descriptor address register
 * -- DMA1_SWCR   : scatter gather channel 1 software control register
 * -- DMA1_UPC    : scatter gather channel 1 unserviced packet count register
 * -- DMA1_PCT    : scatter gather channel 1 packet count threshold register
 * -- DMA1_PWB    : scatter gather channel 1 packet wait bound register
 * -- DMA1_ISR    : dma / scatter gather channel 1 interrupt status register
 * -- DMA1_IER    : dma / scatter gather channel 1 interrupt enable register
 */
#define OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET (0x00000300)
#define OPB_IPIF_TEMPLATE_DMA0_RST_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_DMA0_MIR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_DMA0_DMACR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000004)
#define OPB_IPIF_TEMPLATE_DMA0_SA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000008)
#define OPB_IPIF_TEMPLATE_DMA0_DA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x0000000C)
#define OPB_IPIF_TEMPLATE_DMA0_LENGTH_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000010)
#define OPB_IPIF_TEMPLATE_DMA0_DMASR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000014)
#define OPB_IPIF_TEMPLATE_DMA0_BDA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000018)
#define OPB_IPIF_TEMPLATE_DMA0_SWCR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x0000001C)
#define OPB_IPIF_TEMPLATE_DMA0_UPC_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000020)
#define OPB_IPIF_TEMPLATE_DMA0_PCT_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000024)
#define OPB_IPIF_TEMPLATE_DMA0_PWB_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000028)
#define OPB_IPIF_TEMPLATE_DMA0_ISR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x0000002C)
#define OPB_IPIF_TEMPLATE_DMA0_IER_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000030)
#define OPB_IPIF_TEMPLATE_DMA1_RST_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000040)
#define OPB_IPIF_TEMPLATE_DMA1_MIR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000040)
#define OPB_IPIF_TEMPLATE_DMA1_DMACR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000044)
#define OPB_IPIF_TEMPLATE_DMA1_SA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000048)
#define OPB_IPIF_TEMPLATE_DMA1_DA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x0000004C)
#define OPB_IPIF_TEMPLATE_DMA1_LENGTH_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000050)
#define OPB_IPIF_TEMPLATE_DMA1_DMASR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000054)
#define OPB_IPIF_TEMPLATE_DMA1_BDA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000058)
#define OPB_IPIF_TEMPLATE_DMA1_SWCR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x0000005C)
#define OPB_IPIF_TEMPLATE_DMA1_UPC_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000060)
#define OPB_IPIF_TEMPLATE_DMA1_PCT_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000064)
#define OPB_IPIF_TEMPLATE_DMA1_PWB_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000068)
#define OPB_IPIF_TEMPLATE_DMA1_ISR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x0000006C)
#define OPB_IPIF_TEMPLATE_DMA1_IER_OFFSET (OPB_IPIF_TEMPLATE_IPIF_DMA_SG_SPACE_OFFSET + 0x00000070)

/**
 * IPIF DMA & Scatter Gather Masks
 * -- DMA_SINC_MASK   : dma source address increment
 * -- DMA_DINC_MASK   : dma destination address increment
 * -- DMA_SLOCAL_MASK : dma source address is local
 * -- DMA_DLOCAL_MASK : dma destination address is local
 * -- DMA_SGS_MASK    : dma scatter gather stop as soon as stop condition occurs
 * -- DMA_L_MASK      : dma last (indicate the last of a data packet during scatter gather operation)
 * -- DMA_MODE_MASK   : dma mode (operation mode of individual dma burst transactions)
 * -- DMA_BSY_MASK    : dma busy
 * -- DMA_DBE_MASK    : dma bus error
 * -- DMA_SGL_MASK    : scatter gather last
 * -- DMA_SGE_MASK    : scatter gather enabled by software control
 * -- DMA_SGBSY_MASK  : scatter gather busy
 * -- DMA_SGEND_MASK  : scatter gather end
 * -- DMA_SGDA_MASK   : scatter gather disable acknowledge
 * -- DMA_PWBR_MASK   : scatter gather packet wait bound reached
 * -- DMA_PCTR_MASK   : scatter gather packet count threshold reached
 * -- DMA_PD_MASK     : scatter gather packet done
 * -- DMA_DE_MASK     : dma error
 * -- DMA_DD_MASK     : dma done
 */
#define DMA_SINC_MASK (0x80000000UL)
#define DMA_DINC_MASK (0x40000000UL)
#define DMA_SLOCAL_MASK (0x20000000UL)
#define DMA_DLOCAL_MASK (0x10000000UL)
#define DMA_SGS_MASK (0x08000000UL)
#define DMA_L_MASK (0x02000000UL)
#define DMA_MODE_MASK (0x01000000UL)
#define DMA_BSY_MASK (0x80000000UL)
#define DMA_DBE_MASK (0x40000000UL)
#define DMA_DBT_MASK (0x20000000UL)
#define DMA_SGL_MASK (0x10000000UL)
#define DMA_SGE_MASK (0x80000000UL)
#define DMA_SGBSY_MASK (0x08000000UL)
#define DMA_SGEND_MASK (0x00000040UL)
#define DMA_SGDA_MASK (0x00000020UL)
#define DMA_PWBR_MASK (0x00000010UL)
#define DMA_PCTR_MASK (0x00000008UL)
#define DMA_PD_MASK (0x00000004UL)
#define DMA_DE_MASK (0x00000002UL)
#define DMA_DD_MASK (0x00000001UL)

/**
 * IPIF Read Packet FIFO Register/Data Space Offsets
 * -- RDFIFO_RST   : read packet fifo reset register
 * -- RDFIFO_MIR   : read packet fifo module identification register
 * -- RDFIFO_SR    : read packet fifo status register
 * -- RDFIFO_DATA  : read packet fifo data
 */
#define OPB_IPIF_TEMPLATE_IPIF_RDFIFO_REG_SPACE_OFFSET (0x00000400)
#define OPB_IPIF_TEMPLATE_RDFIFO_RST_OFFSET (OPB_IPIF_TEMPLATE_IPIF_RDFIFO_REG_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_RDFIFO_MIR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_RDFIFO_REG_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_RDFIFO_SR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_RDFIFO_REG_SPACE_OFFSET + 0x00000004)
#define OPB_IPIF_TEMPLATE_IPIF_RDFIFO_DATA_SPACE_OFFSET (0x00000500)
#define OPB_IPIF_TEMPLATE_RDFIFO_DATA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_RDFIFO_DATA_SPACE_OFFSET + 0x00000000)

/**
 * IPIF Read Packet FIFO Masks
 * -- RDFIFO_EMPTY_MASK : read packet fifo empty condition
 * -- RDFIFO_AE_MASK    : read packet fifo almost empty condition
 * -- RDFIFO_DL_MASK    : read packet fifo deadlock condition
 * -- RDFIFO_SCL_MASK   : read packet fifo occupancy scaling enabled
 * -- RDFIFO_WIDTH_MASK : read packet fifo encoded data port width
 * -- RDFIFO_OCC_MASK   : read packet fifo occupancy
 */
#define RDFIFO_EMPTY_MASK (0x80000000UL)
#define RDFIFO_AE_MASK (0x40000000UL)
#define RDFIFO_DL_MASK (0x20000000UL)
#define RDFIFO_SCL_MASK (0x10000000UL)
#define RDFIFO_WIDTH_MASK (0x0E000000UL)
#define RDFIFO_OCC_MASK (0x01FFFFFFUL)

/**
 * IPIF Write Packet FIFO Register/Data Space Offsets
 * -- WRFIFO_RST   : write packet fifo reset register
 * -- WRFIFO_MIR   : write packet fifo module identification register
 * -- WRFIFO_SR    : write packet fifo status register
 * -- WRFIFO_DATA  : write packet fifo data
 */
#define OPB_IPIF_TEMPLATE_IPIF_WRFIFO_REG_SPACE_OFFSET (0x00000600)
#define OPB_IPIF_TEMPLATE_WRFIFO_RST_OFFSET (OPB_IPIF_TEMPLATE_IPIF_WRFIFO_REG_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_WRFIFO_MIR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_WRFIFO_REG_SPACE_OFFSET + 0x00000000)
#define OPB_IPIF_TEMPLATE_WRFIFO_SR_OFFSET (OPB_IPIF_TEMPLATE_IPIF_WRFIFO_REG_SPACE_OFFSET + 0x00000004)
#define OPB_IPIF_TEMPLATE_IPIF_WRFIFO_DATA_SPACE_OFFSET (0x00000700)
#define OPB_IPIF_TEMPLATE_WRFIFO_DATA_OFFSET (OPB_IPIF_TEMPLATE_IPIF_WRFIFO_DATA_SPACE_OFFSET + 0x00000000)

/**
 * IPIF Write Packet FIFO Masks
 * -- WRFIFO_FULL_MASK  : write packet fifo full condition
 * -- WRFIFO_AF_MASK    : write packet fifo almost full condition
 * -- WRFIFO_DL_MASK    : write packet fifo deadlock condition
 * -- WRFIFO_SCL_MASK   : write packet fifo vacancy scaling enabled
 * -- WRFIFO_WIDTH_MASK : write packet fifo encoded data port width
 * -- WRFIFO_VAC_MASK   : write packet fifo vacancy
 */
#define WRFIFO_FULL_MASK (0x80000000UL)
#define WRFIFO_AF_MASK (0x40000000UL)
#define WRFIFO_DL_MASK (0x20000000UL)
#define WRFIFO_SCL_MASK (0x10000000UL)
#define WRFIFO_WIDTH_MASK (0x0E000000UL)
#define WRFIFO_VAC_MASK (0x01FFFFFFUL)

/**************************** Type Definitions *****************************/


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a OPB_IPIF_TEMPLATE register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mWriteReg(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Data)
 *
 */
#define OPB_IPIF_TEMPLATE_mWriteReg(BaseAddress, RegOffset, Data) \
 	XIo_Out32((BaseAddress) + (RegOffset), (Xuint32)(Data))

/**
 *
 * Read a value from a OPB_IPIF_TEMPLATE register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 OPB_IPIF_TEMPLATE_mReadReg(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define OPB_IPIF_TEMPLATE_mReadReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (RegOffset))


/**
 *
 * Reset OPB_IPIF_TEMPLATE via software.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mReset(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mReset(BaseAddress) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_RST_OFFSET), IPIF_RESET)

/**
 *
 * Read module identification information from OPB_IPIF_TEMPLATE device.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	Xuint32 OPB_IPIF_TEMPLATE_mReadMIR(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mReadMIR(BaseAddress) \
 	XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_MIR_OFFSET))

/**
 *
 * Reset DMA channel of OPB_IPIF_TEMPLATE to initial state.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mResetDMA0(Xuint32 BaseAddress)
 * 	void OPB_IPIF_TEMPLATE_mResetDMA1(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mResetDMA0(BaseAddress) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA0_RST_OFFSET), IPIF_RESET)
#define OPB_IPIF_TEMPLATE_mResetDMA1(BaseAddress) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA1_RST_OFFSET), IPIF_RESET)

/**
 *
 * Set/Get DMA control register of OPB_IPIF_TEMPLATE DMA channel.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 * @param   Mask is the control bit(s) to be set.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mSetDMA0Control(Xuint32 BaseAddress, Xuint32 Mask)
 * 	void OPB_IPIF_TEMPLATE_mSetDMA1Control(Xuint32 BaseAddress, Xuint32 Mask)
 * 	Xuint32 OPB_IPIF_TEMPLATE_mGetDMA0Control(Xuint32 BaseAddress)
 * 	Xuint32 OPB_IPIF_TEMPLATE_mGetDMA1Control(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mSetDMA0Control(BaseAddress, Mask) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA0_DMACR_OFFSET), (Xuint32)(Mask))
#define OPB_IPIF_TEMPLATE_mSetDMA1Control(BaseAddress, Mask) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA1_DMACR_OFFSET), (Xuint32)(Mask))
#define OPB_IPIF_TEMPLATE_mGetDMA0Control(BaseAddress) \
 	XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA0_DMACR_OFFSET))
#define OPB_IPIF_TEMPLATE_mGetDMA1Control(BaseAddress) \
 	XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA1_DMACR_OFFSET))

/**
 *
 * Get DMA status register of OPB_IPIF_TEMPLATE DMA channel.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  Status is the DMA channel status.
 *
 * @note
 * C-style signature:
 * 	Xuint32 OPB_IPIF_TEMPLATE_mGetDMA0Status(Xuint32 BaseAddress)
 * 	Xuint32 OPB_IPIF_TEMPLATE_mGetDMA1Status(Xuint32 BaseAddress)
 * 	bool OPB_IPIF_TEMPLATE_mDMA0Error(Xuint32 BaseAddress)
 * 	bool OPB_IPIF_TEMPLATE_mDMA1Error(Xuint32 BaseAddress)
 * 	bool OPB_IPIF_TEMPLATE_mDMA0Done(Xuint32 BaseAddress)
 * 	bool OPB_IPIF_TEMPLATE_mDMA1Done(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mGetDMA0Status(BaseAddress) \
 	XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA0_DMASR_OFFSET))
#define OPB_IPIF_TEMPLATE_mGetDMA1Status(BaseAddress) \
 	XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA1_DMASR_OFFSET))
#define OPB_IPIF_TEMPLATE_mDMA0Error(BaseAddress) \
 	((XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA0_ISR_OFFSET)) & DMA_DE_MASK) == DMA_DE_MASK)
#define OPB_IPIF_TEMPLATE_mDMA1Error(BaseAddress) \
 	((XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA1_ISR_OFFSET)) & DMA_DE_MASK) == DMA_DE_MASK)
#define OPB_IPIF_TEMPLATE_mDMA0Done(BaseAddress) \
 	((XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA0_ISR_OFFSET)) & DMA_DD_MASK) == DMA_DD_MASK)
#define OPB_IPIF_TEMPLATE_mDMA1Done(BaseAddress) \
 	((XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_DMA1_ISR_OFFSET)) & DMA_DD_MASK) == DMA_DD_MASK)

/**
 *
 * Reset read packet FIFO of OPB_IPIF_TEMPLATE to its initial state.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mResetReadFIFO(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mResetReadFIFO(BaseAddress) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_RDFIFO_RST_OFFSET), IPIF_RESET)

/**
 *
 * Check status of OPB_IPIF_TEMPLATE read packet FIFO module.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  Status is the result of status checking.
 *
 * @note
 * C-style signature:
 * 	bool OPB_IPIF_TEMPLATE_mReadFIFOEmpty(Xuint32 BaseAddress)
 * 	Xuint32 OPB_IPIF_TEMPLATE_mReadFIFOOccupancy(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mReadFIFOEmpty(BaseAddress) \
 	((XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_RDFIFO_SR_OFFSET)) & RDFIFO_EMPTY_MASK) == RDFIFO_EMPTY_MASK)
#define OPB_IPIF_TEMPLATE_mReadFIFOOccupancy(BaseAddress) \
 	(XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_RDFIFO_SR_OFFSET)) & RDFIFO_OCC_MASK)

/**
 *
 * Read data from OPB_IPIF_TEMPLATE read packet FIFO module.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  Data is the data from the read packet FIFO.
 *
 * @note
 * C-style signature:
 * 	Xuint32 OPB_IPIF_TEMPLATE_mReadFromFIFO(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mReadFromFIFO(BaseAddress) \
 	XIo_In32((BaseAddress) + (OPB_IPIF_TEMPLATE_RDFIFO_DATA_OFFSET))

/**
 *
 * Reset write packet FIFO of OPB_IPIF_TEMPLATE to its initial state.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mResetWriteFIFO(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mResetWriteFIFO(BaseAddress) \
 	XIo_Out32((BaseAddress)+(OPB_IPIF_TEMPLATE_WRFIFO_RST_OFFSET), IPIF_RESET)

/**
 *
 * Check status of OPB_IPIF_TEMPLATE write packet FIFO module.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  Status is the result of status checking.
 *
 * @note
 * C-style signature:
 * 	bool OPB_IPIF_TEMPLATE_mWriteFIFOFull(Xuint32 BaseAddress)
 * 	Xuint32 OPB_IPIF_TEMPLATE_mWriteFIFOVacancy(Xuint32 BaseAddress)
 *
 */
#define OPB_IPIF_TEMPLATE_mWriteFIFOFull(BaseAddress) \
 	((XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_WRFIFO_SR_OFFSET)) & WRFIFO_FULL_MASK) == WRFIFO_FULL_MASK)
#define OPB_IPIF_TEMPLATE_mWriteFIFOVacancy(BaseAddress) \
 	(XIo_In32((BaseAddress)+(OPB_IPIF_TEMPLATE_WRFIFO_SR_OFFSET)) & WRFIFO_VAC_MASK)

/**
 *
 * Write data to OPB_IPIF_TEMPLATE write packet FIFO module.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 * @param   Data is the value to be written to write packet FIFO.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void OPB_IPIF_TEMPLATE_mWriteToFIFO(Xuint32 BaseAddress, Xuint32 Data)
 *
 */
#define OPB_IPIF_TEMPLATE_mWriteToFIFO(BaseAddress, Data) \
 	XIo_Out32((BaseAddress) + (OPB_IPIF_TEMPLATE_WRFIFO_DATA_OFFSET), (Xuint32)(Data))

/************************** Function Prototypes ****************************/


/**
 *
 * DMA transfer between source address and destination address, either the source or the destination address.
 * must be local to the OPB_IPIF_TEMPLATE device.
 *
 * @param   BaseAddress is the base address of the OPB_IPIF_TEMPLATE device.
 * @param   SrcAddress is the source address from which the data will be fetched.
 * @param   DstAddress is the destination address to which the data will be stored.
 * @param   ByteCount is the number of bytes to be sent.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void OPB_IPIF_TEMPLATE_DMA0Transfer(Xuint32 BaseAddress, Xuint32 SrcAddress, Xuint32 DstAddress, Xuint32 ByteCount);
void OPB_IPIF_TEMPLATE_DMA1Transfer(Xuint32 BaseAddress, Xuint32 SrcAddress, Xuint32 DstAddress, Xuint32 ByteCount);

/**
 *
 * Enable all possible interrupts from OPB_IPIF_TEMPLATE device.
 *
 * @param   baseaddr_p is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void OPB_IPIF_TEMPLATE_EnableInterrupt(void * baseaddr_p);

/**
 *
 * Example interrupt controller handler.
 *
 * @param   baseaddr_p is the base address of the OPB_IPIF_TEMPLATE device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void OPB_IPIF_TEMPLATE_Intr_DefaultHandler(void * baseaddr_p);

/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the OPB_IPIF_TEMPLATE instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus OPB_IPIF_TEMPLATE_SelfTest(void * baseaddr_p);


// CUSTOM FUNCTIONS DEFINITION:
/**
 * A Function that will reset the Custom IP, Write 1024 Bytes to the Write FIFO using Channel 0 DMA, is written 
 *  and added to the drivers.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   Xuint32 SrcAddress is the address of the buffer memory where Data is located.
 * 
 * @return 
 *
 *    - XST_SUCCESS   if the transfer is OK.
 *    - Service Error Code is not.
 *
 */
XStatus OPB_IPIF_TEMPLATE_Write1KB(Xuint32 SrcAddress, Xuint32 BaseAddress);


#endif // OPB_IPIF_TEMPLATE_H
