//////////////////////////////////////////////////////////////////////////////
// Filename:          C:\xapp967/drivers/opb_ipif_template_v1_00_a/src/opb_ipif_template_selftest.c
// Version:           1.00.a
// Description:       Contains a diagnostic self-test function for the opb_ipif_template driver
// Date:              Thu Oct 19 19:24:20 2006 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////


/***************************** Include Files *******************************/

#include "opb_ipif_template.h"

/************************** Constant Definitions ***************************/

#define OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE  128 /* Size of buffer (for transfer test) in bytes */

/************************** Variable Definitions ****************************/

static Xuint8 SrcBuffer[OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE];   /* Source buffer      */
static Xuint8 DstBuffer[OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE];   /* Destination buffer */

/************************** Function Definitions ***************************/

/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the OPB_IPIF_TEMPLATE instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus OPB_IPIF_TEMPLATE_SelfTest(void * baseaddr_p)
{
   int     Index;
   Xuint32 baseaddr;
   Xuint8  Reg8Value;
   Xuint16 Reg16Value;
   Xuint32 Reg32Value;
   Xuint64 Reg64Value;

   /*
    * Assert the argument
    */
   XASSERT_NONVOID(baseaddr_p != XNULL);
   baseaddr = (Xuint32) baseaddr_p;

   xil_printf("******************************\n\r");
   xil_printf("* User Peripheral Self Test\n\r");
   xil_printf("******************************\n\n\r");

   /*
    * Reset the device to get it back to its default state and get module identification value
    */
   xil_printf("RST/MIR test...\n\r");
   OPB_IPIF_TEMPLATE_mReset(baseaddr);
   xil_printf("   - write 0x%08x to software reset register \n\r", IPIF_RESET);
   Reg32Value = OPB_IPIF_TEMPLATE_mReadMIR(baseaddr);
   if ( Reg32Value == 0x20070301 )
   {
      xil_printf("   - read 0x%08x (expected) from module identification register \n\r", Reg32Value);
      xil_printf("   - RST/MIR write/read passed\n\n\r");
   }
   else
   {
      xil_printf("   - read 0x%08x (unexpected) from module identification register, expect 0x20070301 \n\r", Reg32Value);
      xil_printf("   - RST/MIR write/read failed\n\n\r");
      return XST_FAILURE;
   }

   /*
    * Write to write packet FIFO and read back from read packet FIFO
    */
   xil_printf("Packet FIFO test...\n\r");
   xil_printf("   - reset write packet FIFO to initial state \n\r");
   OPB_IPIF_TEMPLATE_mResetWriteFIFO(baseaddr);
   xil_printf("   - reset read packet FIFO to initial state \n\r");
   OPB_IPIF_TEMPLATE_mResetReadFIFO(baseaddr);
   xil_printf("   - push data to write packet FIFO \n\r");
   for ( Index = 0; Index < 4; Index++ )
   {
      xil_printf("     0x%08x \n\r", Index);
      OPB_IPIF_TEMPLATE_mWriteToFIFO(baseaddr, Index);
   }
   xil_printf("   - write packet FIFO is %s \n\r", ( OPB_IPIF_TEMPLATE_mWriteFIFOFull(baseaddr) ? "full" : "not full" ));
   /* 
    * FIFO example in user logic will loop WrFIFO data back to RdFIFO,
    * so we get number of entries from RdFIFO instead of WrFIFO.
    * Reg32Value = OPB_IPIF_TEMPLATE_mWriteFIFOVacancy(baseaddr);
    */
   Reg32Value = OPB_IPIF_TEMPLATE_mReadFIFOOccupancy(baseaddr);
   xil_printf("   - number of entries is %s %d \n\r", ( Reg32Value == (Xuint32) 4 ? "expected" : "unexpected" ), Reg32Value);
   xil_printf("   - pop data out from read packet FIFO \n\r");
   for ( Index = 0; Index < 4; Index++ )
   {
      Reg32Value = OPB_IPIF_TEMPLATE_mReadFromFIFO(baseaddr);
      xil_printf("     0x%08x \n\r", Reg32Value);
      if ( Reg32Value != (Xuint32) Index )
      {
         xil_printf("   - unexpected value read from read packet FIFO \n\r");
         xil_printf("   - write/read packet FIFO failed \n\r");
         OPB_IPIF_TEMPLATE_mResetWriteFIFO(baseaddr);
         OPB_IPIF_TEMPLATE_mResetReadFIFO(baseaddr);
         return XST_FAILURE;
      }
   }
   xil_printf("   - read packet FIFO is %s \n\r", ( OPB_IPIF_TEMPLATE_mReadFIFOEmpty(baseaddr) ? "empty" : "not empty" ));
   Reg32Value = OPB_IPIF_TEMPLATE_mReadFIFOOccupancy(baseaddr);
   xil_printf("   - number of entries is %s %d \n\r", ( Reg32Value == (Xuint32) 0 ? "expected" : "unexpected" ), Reg32Value);
   OPB_IPIF_TEMPLATE_mResetWriteFIFO(baseaddr);
   OPB_IPIF_TEMPLATE_mResetReadFIFO(baseaddr);
   xil_printf("   - write/read packet FIFO passed \n\n\r");

   /*
    * Enable all possible interrupts and clear interrupt status register(s)
    */
   xil_printf("Interrupt controller test...\n\r");
   Reg32Value = OPB_IPIF_TEMPLATE_mReadReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET);
   xil_printf("   - IP (user logic) interrupt status : 0x%08x \n\r", Reg32Value);
   xil_printf("   - clear IP (user logic) interrupt status register\n\r");
   OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_ISR_OFFSET, Reg32Value);
   Reg32Value = OPB_IPIF_TEMPLATE_mReadReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DISR_OFFSET);
   xil_printf("   - Device (peripheral) interrupt status : 0x%08x \n\r", Reg32Value);
   xil_printf("   - clear Device (peripheral) interrupt status register\n\r");
   OPB_IPIF_TEMPLATE_mWriteReg(baseaddr, OPB_IPIF_TEMPLATE_INTR_DISR_OFFSET, Reg32Value);
   xil_printf("   - enable all possible interrupt(s)\n\r");
   OPB_IPIF_TEMPLATE_EnableInterrupt(baseaddr_p);
   xil_printf("   - write/read interrupt register passed \n\n\r");

   /*
    * Setup DMA channel 0 to receive data from source buffer to local FIFO and
    * DMA channel 1 to send data from local FIFO to destination buffer
    */
   xil_printf("Simple DMA test...\n\r");
   xil_printf("   - source buffer address is 0x%08x \n\r", SrcBuffer);
   xil_printf("   - destination buffer address is 0x%08x \n\r", DstBuffer);
   xil_printf("   - initialize the source buffer bytes with a pattern \n\r");
   xil_printf("   - initialize the destination buffer bytes to zero \n\r");
   for ( Index = 0; Index < OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE; Index++ )
   {
      SrcBuffer[Index] = Index;
      DstBuffer[Index] = 0;
   }

   xil_printf("   - \n\r");
   xil_printf("   - reset DMA channel 0 to initial state \n\r");
   OPB_IPIF_TEMPLATE_mResetDMA0(baseaddr);
   xil_printf("   - setup DMA channel 0 to transfer data from remote source to local FIFO \n\r");
   OPB_IPIF_TEMPLATE_mSetDMA0Control(baseaddr, DMA_SINC_MASK | DMA_DLOCAL_MASK);
   xil_printf("   - start DMA channel 0 to receive bytes from the source buffer \n\r");
   OPB_IPIF_TEMPLATE_DMA0Transfer(baseaddr, (Xuint32) SrcBuffer, baseaddr+OPB_IPIF_TEMPLATE_WRFIFO_DATA_OFFSET, OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE);
   while ( ! OPB_IPIF_TEMPLATE_mDMA0Done(baseaddr) ) {}
   xil_printf("   - DMA channel 0 transfer completed \n\r");
   if ( OPB_IPIF_TEMPLATE_mDMA0Error(baseaddr) )
   {
      xil_printf("   - DMA channel 0 transfer error \n\r");
      xil_printf("   - DMA failed \n\r");
      return XST_FAILURE;
   }
   xil_printf("   - reset DMA channel 1 to initial state \n\r");
   OPB_IPIF_TEMPLATE_mResetDMA1(baseaddr);
   xil_printf("   - setup DMA channel 1 to transfer data from local FIFO to remote destination \n\r");
   OPB_IPIF_TEMPLATE_mSetDMA1Control(baseaddr, DMA_DINC_MASK | DMA_SLOCAL_MASK);
   xil_printf("   - start DMA channel 1 to send bytes to the destination buffer \n\r");
   OPB_IPIF_TEMPLATE_DMA1Transfer(baseaddr, baseaddr+OPB_IPIF_TEMPLATE_RDFIFO_DATA_OFFSET, (Xuint32) DstBuffer, OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE);
   while ( ! OPB_IPIF_TEMPLATE_mDMA1Done(baseaddr) ) {}
   xil_printf("   - DMA channel 1 transfer completed \n\r");
   if ( OPB_IPIF_TEMPLATE_mDMA1Error(baseaddr) )
   {
      xil_printf("   - DMA channel 1 transfer error \n\r");
      xil_printf("   - DMA failed \n\r");
      return XST_FAILURE;
   }
   for ( Index = 0; Index < OPB_IPIF_TEMPLATE_SELFTEST_BUFSIZE; Index++ )
   {
      if ( DstBuffer[Index] != SrcBuffer[Index] )
      {
         xil_printf("   - destination buffer's contents are different from the source buffer \n\r");
         xil_printf("   - DMA failed \n\r");
         return XST_FAILURE;
      }
   }
   xil_printf("   - destination buffer's contents are same as the source buffer \n\r");
   OPB_IPIF_TEMPLATE_mResetDMA0(baseaddr);
   OPB_IPIF_TEMPLATE_mResetDMA1(baseaddr);
   xil_printf("   - Simple DMA transfer passed \n\n\r");

   return XST_SUCCESS;
}
